from flask import Flask, render_template, request, redirect, url_for, session, jsonify, send_from_directory
import os
import logging
import math
import json
import zipfile
import traceback
import numpy as np
import pandas as pd
from datetime import datetime
from functools import wraps

# --- App Initialization ---
app = Flask(__name__, static_folder="static", static_url_path="/static")
app.secret_key = "a-very-secret-key-that-you-should-change"

# --- Logging Configuration ---
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# --- Directory and Path Setup ---
BASE_DIR = os.path.dirname(os.path.abspath(__file__))
GENERATED_DIR = os.path.join(BASE_DIR, "static", "generated")
os.makedirs(GENERATED_DIR, exist_ok=True)

# --- User Credentials ---
USERNAME = "digitalhuskita"
PASSWORD = "lolpop789123"

# --- Authentication ---
def login_required(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        if not session.get("logged_in"):
            return redirect(url_for("login"))
        return f(*args, **kwargs)
    return wrapper

@app.route("/login", methods=["GET", "POST"])
def login():
    if request.method == "POST":
        user = request.form.get("username")
        pw = request.form.get("password")
        if user == USERNAME and pw == PASSWORD:
            session["logged_in"] = True
            return redirect(url_for("index"))
        return render_template("login.html", error="অবৈধ ব্যবহারকারীর নাম বা পাসওয়ার্ড")
    return render_template("login.html")

@app.route("/logout")
def logout():
    session.pop("logged_in", None)
    return redirect(url_for("login"))

# --- Core Application Routes ---
@app.route("/")
@login_required
def index():
    try:
        files_with_details = []
        
        if not os.path.isdir(GENERATED_DIR):
            logging.error(f"Generated directory not found: {GENERATED_DIR}")
            raise FileNotFoundError(f"Server configuration error: Directory '{GENERATED_DIR}' not found.")

        for filename in os.listdir(GENERATED_DIR):
            if filename.endswith('.zip'):
                file_path = os.path.join(GENERATED_DIR, filename)
                try:
                    stat = os.stat(file_path)
                    files_with_details.append({
                        'name': filename,
                        'size': f"{stat.st_size / 1024:.2f} KB",
                        'created_at': datetime.fromtimestamp(stat.st_ctime).strftime('%Y-%m-%d %H:%M')
                    })
                except (FileNotFoundError, PermissionError) as e:
                    logging.warning(f"Could not read metadata for {filename}: {e}")
                    continue
        
        files_with_details.sort(key=lambda x: x['created_at'], reverse=True)
        
        return render_template("form_ajax.html", files=files_with_details)
    except Exception as e:
        error_trace = traceback.format_exc()
        logging.error(f"A critical error occurred in the index route:\n{error_trace}")
        return "A critical server error occurred. Please check the server logs.", 500


@app.route("/generate", methods=["POST"])
@login_required
def generate():
    try:
        form_data = request.form
        required_fields = ["center_lat", "center_lon", "radius_min", "radius_max", "radius_step", "prefix"]
        
        for field in required_fields:
            if not form_data.get(field):
                raise ValueError(f"'{field}' একটি প্রয়োজনীয় ফিল্ড এবং এটি খালি রাখা যাবে না।")

        try:
            lat = float(form_data["center_lat"])
            lon = float(form_data["center_lon"])
            radius_min = float(form_data["radius_min"])
            radius_max = float(form_data["radius_max"])
            radius_step = float(form_data["radius_step"])
        except (ValueError, TypeError):
            raise ValueError("অক্ষাংশ, দ্রাঘিমাংশ এবং ব্যাসার্ধ ফিল্ডগুলিতে শুধুমাত্র সংখ্যা থাকতে হবে।")

        fields = {k: form_data.get(k, "") for k in [
            "name", "email", "website", "phone", "address",
            "description", "keywords", "facebook", "instagram", "other"]}
        prefix = form_data["prefix"].replace(" ", "_")

        total_files = 50
        points_per_ring = 1000
        rings_per_file = 2
        required_rings = total_files * rings_per_file
        
        if radius_step <= 0:
            raise ValueError("ব্যাসার্ধের ধাপ অবশ্যই একটি ধনাত্মক সংখ্যা হতে হবে।")

        radii = np.arange(radius_min, radius_max + radius_step, radius_step)
        
        # *** সমাধান: ধাপ সংখ্যা পরীক্ষা করার কঠোর নিয়মটি আবার যোগ করা হয়েছে ***
        if len(radii) < required_rings:
            raise ValueError(f"অপর্যাপ্ত ব্যাসার্ধের ধাপ নেই। আপনার সেটিংসে {len(radii)}টি রিং তৈরি হচ্ছে, কিন্তু {required_rings}টি প্রয়োজন।")

        km_lat = 1 / 111.0
        km_lon = 1 / (111.0 * math.cos(math.radians(lat)))
        
        for item in os.listdir(GENERATED_DIR):
            if item.endswith(".csv") and item.startswith(prefix):
                os.remove(os.path.join(GENERATED_DIR, item))

        for i in range(total_files):
            rows = []
            for j in range(rings_per_file):
                idx = i * rings_per_file + j
                if idx >= len(radii):
                    break 
                r = radii[idx]
                angles = np.linspace(0, 2 * math.pi, points_per_ring, endpoint=False)
                for a in angles:
                    lat_pt = lat + r * km_lat * math.cos(a)
                    lon_pt = lon + r * km_lon * math.sin(a)
                    rows.append({**fields, "Latitude": lat_pt, "Longitude": lon_pt, "Distance_km": r})
            
            if not rows:
                continue

            df = pd.DataFrame(rows)
            filename = f"{prefix}_{i+1}.csv"
            df.to_csv(os.path.join(GENERATED_DIR, filename), index=False)

        return jsonify(status="success")

    except Exception as e:
        logging.exception("Generate error")
        return jsonify(status="error", message=str(e)), 400

@app.route("/zip-all", methods=["POST"])
@login_required
def zip_all():
    try:
        prefix = request.form.get("prefix", "output").replace(" ", "_")
        if not prefix:
            return jsonify(status="error", message="Prefix is required for zipping."), 400
            
        zip_path = os.path.join(GENERATED_DIR, f"{prefix}_bundle.zip")
        
        files_to_zip = [fn for fn in os.listdir(GENERATED_DIR) if fn.endswith(".csv") and fn.startswith(prefix)]
        
        if not files_to_zip:
            return jsonify(status="error", message=f"No CSV files found with prefix '{prefix}' to zip."), 404

        with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as z:
            for fn in files_to_zip:
                z.write(os.path.join(GENERATED_DIR, fn), arcname=fn)
        
        for fn in files_to_zip:
            os.remove(os.path.join(GENERATED_DIR, fn))
                
        return jsonify(status="success", zipfile=os.path.basename(zip_path))
    except Exception as e:
        logging.exception("Zip error")
        return jsonify(status="error", message=str(e)), 500

@app.route("/download/<path:filename>")
@login_required
def download(filename):
    return send_from_directory(GENERATED_DIR, filename, as_attachment=True)

@app.route("/delete/<string:filename>", methods=["POST"])
@login_required
def delete(filename):
    try:
        file_path = os.path.join(GENERATED_DIR, filename)
        if os.path.exists(file_path):
            os.remove(file_path)
            return jsonify(status="success")
        else:
            return jsonify(status="error", message="File not found."), 404
    except Exception as e:
        logging.exception("Delete error")
        return jsonify(status="error", message=str(e)), 500

# --- Main Execution ---
if __name__ == "__main__":
    app.run(debug=True)
